@@ -1,3 +1,14 @@
+/************************************************************************
+ * Licensed under Public Domain (CC0)                                    *
+ *                                                                       *
+ * To the extent possible under law, the person who associated CC0 with  *
+ * this code has waived all copyright and related or neighboring         *
+ * rights to this code.                                                  *
+ *                                                                       *
+ * You should have received a copy of the CC0 legalcode along with this  *
+ * work. If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.*
+ ************************************************************************/
+
 package org.reactivestreams.tck;
 
 import org.reactivestreams.Publisher;
@@ -8,13 +19,16 @@ import org.reactivestreams.tck.support.TestException;
 import org.testng.Assert;
 import org.testng.annotations.Test;
 
+import java.util.Collection;
 import java.util.Random;
+import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.AtomicLong;
 
 /**
 * Validates that the TCK's {@link org.reactivestreams.tck.PublisherVerification} fails with nice human readable errors.
@@ -241,6 +255,7 @@ public class PublisherVerificationTest extends TCKVerificationSupport {
                 // but keep signalling data if more demand comes in anyway!
                 if (!completed) {
                   s.onComplete();
+                  completed = true;
                 }
 
               }
@@ -352,6 +367,21 @@ public class PublisherVerificationTest extends TCKVerificationSupport {
     noopPublisherVerification().optional_spec111_maySupportMultiSubscribe();
   }
 
+  @Test
+  public void optional_spec111_registeredSubscribersMustReceiveOnNextOrOnCompleteSignals_beSkippedWhenMultipleSubscribersNotSupported() throws Throwable {
+    requireTestSkip(new ThrowingRunnable() {
+      @Override
+      public void run() throws Throwable {
+        multiSubscribersPublisherVerification(true).optional_spec111_registeredSubscribersMustReceiveOnNextOrOnCompleteSignals();
+      }
+    }, "Unexpected additional subscriber");
+  }
+
+  @Test
+  public void optional_spec111_registeredSubscribersMustReceiveOnNextOrOnCompleteSignals_shouldPass() throws Throwable {
+    multiSubscribersPublisherVerification(false).optional_spec111_registeredSubscribersMustReceiveOnNextOrOnCompleteSignals();
+  }
+
   @Test
   public void optional_spec111_multicast_mustProduceTheSameElementsInTheSameSequenceToAllOfItsSubscribersWhenRequestingManyUpfront_shouldFailBy_expectingOnError() throws Throwable {
     requireTestFailure(new ThrowingRunnable() {
@@ -451,6 +481,27 @@ public class PublisherVerificationTest extends TCKVerificationSupport {
     }, "Expected onError");
   }
 
+  @Test
+  public void required_spec309_requestZeroMustSignalIllegalArgumentException_shouldPass() throws Throwable {
+    customPublisherVerification(new Publisher<Integer>() {
+      @Override
+      public void subscribe(final Subscriber<? super Integer> s) {
+        s.onSubscribe(new Subscription() {
+          @Override
+          public void request(long n) {
+            // we error out with any message, it does not have to contain any specific wording
+            if (n <= 0) s.onError(new IllegalArgumentException("Illegal request value detected!"));
+          }
+
+          @Override
+          public void cancel() {
+            // noop
+          }
+        });
+      }
+    }).required_spec309_requestZeroMustSignalIllegalArgumentException();
+  }
+
   @Test
   public void required_spec309_requestNegativeNumberMustSignalIllegalArgumentException_shouldFailBy_expectingOnError() throws Throwable {
     requireTestFailure(new ThrowingRunnable() {
@@ -704,6 +755,73 @@ public class PublisherVerificationTest extends TCKVerificationSupport {
     };
   }
 
+  /**
+   * Verification using a Publisher that supports multiple subscribers
+   * @param shouldBlowUp if true {@link RuntimeException} will be thrown during second subscription.
+   */
+  final PublisherVerification<Integer> multiSubscribersPublisherVerification(final boolean shouldBlowUp) {
+    return new PublisherVerification<Integer>(newTestEnvironment()) {
+
+      @Override
+      public Publisher<Integer> createPublisher(final long elements) {
+        return new Publisher<Integer>() {
+
+          private final Collection<CancelableSubscription> subscriptions = new CopyOnWriteArrayList<CancelableSubscription>();
+          private final AtomicLong source = new AtomicLong(elements);
+
+          @Override
+          public void subscribe(Subscriber<? super Integer> s) {
+            // onSubscribe first
+            CancelableSubscription subscription = new CancelableSubscription(s);
+            s.onSubscribe(subscription);
+            if (shouldBlowUp && !subscriptions.isEmpty()) {
+              s.onError(new RuntimeException("Unexpected additional subscriber"));
+            } else {
+              subscriptions.add(subscription);
+            }
+          }
+
+          class CancelableSubscription implements Subscription {
+
+            final AtomicBoolean canceled = new AtomicBoolean();
+            Subscriber<? super Integer> subscriber;
+
+            CancelableSubscription(Subscriber<? super Integer> subscriber) {
+              this.subscriber = subscriber;
+            }
+
+            @Override
+            public void request(long n) {
+              if (!canceled.get()) {
+                for (long i = 0; i < n; i++) {
+                  if (source.getAndDecrement() < 0) {
+                    canceled.set(true);
+                    subscriber.onComplete();
+                  } else {
+                    subscriber.onNext((int) i);
+                  }
+                }
+              }
+            }
+
+            @Override
+            public void cancel() {
+              canceled.set(true);
+              subscriber = null;
+              subscriptions.remove(this);
+            }
+          }
+
+        };
+      }
+
+      @Override
+      public Publisher<Integer> createFailedPublisher() {
+        return SKIP;
+      }
+    };
+  }
+
   /**
    * Verification using a Publisher that publishes elements even with no demand available
    */
